@@ -4,7 +4,7 @@ use vars qw[@ISA $VERSION];
 use GD::SecurityImage::Styles;
 
 @ISA     = qw(GD::SecurityImage::Styles);
-$VERSION = "1.33";
+$VERSION = '1.5';
 
 sub import {
    # load the drawing interface
@@ -44,8 +44,33 @@ sub new {
                bgcolor    => $opt{bgcolor}             || [255, 255, 255],
                send_ctobg => $opt{send_ctobg}          || 0,
                frame      => defined($opt{frame}) ? $opt{frame} : 1,
+               scramble   => $opt{scramble}            || 0,
+               angle      => $opt{angle}               || 0,
+               thickness  => $opt{thickness}           || 0,
+               _ANGLES_   => [], # angle list for scrambled images
    );
+
+   $self->{_RNDMAX_} = $options{rndmax}; 
+
    $self->{$_} = $options{$_} foreach keys %options;
+   if($self->{angle}) { # validate angle
+      $self->{angle} = 360 + $self->{angle} if $self->{angle} < 0;
+      if($self->{angle} > 360) {
+         die "Angle parameter can take values in the range -360..360";
+      }
+   }
+
+   if ($self->{scramble}) {
+      if ($self->{angle}) {
+         # Does the user want a fixed angle?
+         push @{ $self->{_ANGLES_} }, $self->{angle};
+      } else {
+         # Generate angle range. The reason for hardcoding these is; 
+         # it'll be less random for 0..60 range
+         push @{ $self->{_ANGLES_} }, (0,5,8,15,22,26,29,33,35,36,40,43,45,53,56);
+         push @{ $self->{_ANGLES_} }, map {360 - $_} @{ $self->{_ANGLES_} }; # push negatives
+      }
+   }
    $self->init;
    return $self;
 }
@@ -56,6 +81,14 @@ sub gdf {
    return $self->gdfx(@_);
 }
 
+sub random_angle {
+   my $self   = shift;
+   my @angles = @{ $self->{_ANGLES_} };
+   my @r;
+   push @r, $angles[int rand @angles] for 0..$#angles;
+   return $r[int rand @r];
+}
+
 sub random_str { shift->{_RANDOM_NUMBER_} }
 
 sub random {
@@ -89,6 +122,12 @@ sub create {
    $self->{send_ctobg} = 0 if $style eq 'box'; # disable for that style
 
    $self->{_COLOR_} = \%color; # set the color hash
+
+   # be a smart module and auto-disable ttf if we are under a prehistoric GD
+   unless ($self->{IS_MAGICK}) {
+      $method = 'normal' if defined $GD::VERSION and $GD::VERSION < 1.20;
+   }
+
    if($method eq 'normal' and not $self->{gd_font}) {
       $self->{gd_font} = $self->gdf('giant');
    }
@@ -146,7 +185,7 @@ __END__
 
 =head1 NAME
 
-GD::SecurityImage - Create a security image with a random string on it.
+GD::SecurityImage - Security image (captcha) generator.
 
 =head1 SYNOPSIS
 
@@ -162,10 +201,11 @@ GD::SecurityImage - Create a security image with a random string on it.
    my($image_data, $mime_type, $random_number) = $image->out;
 
    # use external ttf font
-   my $image = GD::SecurityImage->new(width  => 100,
-                                      height => 40,
-                                      lines  => 10,
-                                      font   => "/absolute/path/to/your.ttf");
+   my $image = GD::SecurityImage->new(width    => 100,
+                                      height   => 40,
+                                      lines    => 10,
+                                      font     => "/absolute/path/to/your.ttf",
+                                      scramble => 1);
       $image->random($your_random_str);
       $image->create(ttf => 'default');
       $image->particle;
@@ -176,7 +216,7 @@ or you can just say (all public methods can be chained)
    my($image, $type, $rnd) = GD::SecurityImage->new->random->create->particle->out;
 
 to create a security image with the default settings. But that may not be 
-usefull.
+useful.
 
 If you C<require> the module, you B<must> import it also:
 
@@ -221,10 +261,11 @@ necessary sub modules.
 
 The (so called) I<"Security Images"> are so popular. Most internet 
 software use these in their registration screens to block robot programs
-(which may register tons of  fake member accounts). This module gives
-you a basic interface to create such an image. The final output is
-the actual graphic data, the mime type of the graphic and the created
-random string.
+(which may register tons of  fake member accounts). Security images are
+basicly, graphical CAPTCHAs (Completely Automated Public Turing Test to 
+Tell Computers and Humans Apart). This module gives you a basic interface 
+to create such an image. The final output is the actual graphic data, 
+the mime type of the graphic and the created random string.
 
 The module also has some I<"styles"> that are used to create the background 
 of the image.
@@ -292,6 +333,27 @@ background and the lines will pass over it.
 If has a true value, a frame will be added around the image. This
 option is enabled by default.
 
+=item scramble
+
+If set, the characters will be scrambled. If you enable this option,
+be sure to use a wider image, since the characters will be separeted 
+with three spaces.
+
+=item angle
+
+Sets the angle for scrambled characters. Beware that, if you pass
+an C<angle> parameter, the characters in your random string will have
+a fixed angle. If you do not set an C<angle> parameter, the angle(s)
+will be random.
+
+Unlike the GD interface, C<angle> is in C<degree>s and can take values 
+between C<0> and C<360>.
+
+=item thickness
+
+Sets the line drawing width. Can take numerical values. 
+Default values are C<1> for GD and C<0.6> for Image:Magick.
+
 =item rndmax
 
 The length of the random string. Default value is C<6>.
@@ -412,7 +474,11 @@ C<out> method accepts arguments:
 
    @data = $image->out(%args);
 
-currently, you can only set output format with the C<force> key:
+=over 4
+
+=item force
+
+You can set the output format with the C<force> parameter:
 
    @data = $image->out(force => 'png');
 
@@ -420,11 +486,34 @@ If C<png> is supported by the interface (via C<GD> or C<Image::Magick>);
 you'll get a png image, if the interface does not support this format, 
 C<out()> method will use it's default configuration.
 
-Currently, you can not define compression values for the formats that 
-support it (eg: jpeg, png), but you can use L<raw|/raw> method instead 
-of C<out> (for a direct communication with the graphic library -- but 
-probably you do not want to do that, future versions may implement 
-this feature).
+=item compress
+
+And with the C<compress> parameter, you can define the compression 
+for C<png> and quality for C<jpeg>:
+
+   @data = $image->out(force => 'png' , compress => 1);
+   @data = $image->out(force => 'jpeg', compress => 100);
+
+When you use C<compress> with C<png> format, the value of C<compress>
+is ignored and it is only checked if it has a true value. With C<png>
+the compression will always be C<9> (maximum compression). eg:
+
+   @data = $image->out(force => 'png' , compress => 1);
+   @data = $image->out(force => 'png' , compress => 3);
+   @data = $image->out(force => 'png' , compress => 5);
+   @data = $image->out(force => 'png' , compress => 1500);
+
+All will default to C<9>. But this will disable compression:
+
+   @data = $image->out(force => 'png' , compress => 0);
+
+But the behaviour changes if the format is C<jpeg>; the value of
+C<compress> will be used for C<jpeg> quality; which is in the range 
+C<1..100>.
+
+Compression and quality operations are disabled by default.
+
+=back
 
 =head2 raw
 
@@ -444,7 +533,11 @@ use another output format like C<png>.
 
 =head1 EXAMPLES
 
-See the tests in the distribution.
+See the tests in the distribution. Also see the demo program 
+"eg/demo.pl" for an C<Apache::Session> implementation of C<GD::SecurityImage>.
+
+Download the distribution from a CPAN mirror near you, if you don't have 
+the files.
 
 =head1 ERROR HANDLING
 
@@ -477,11 +570,22 @@ The internal random code generator is used B<only> for demonstration
 purposes for this module. It may not be I<effective>. You must supply 
 your own random code and use this module to display it.
 
+=item [GD] png compression
+
+Support for compression level argument to png() added in v2.07. If
+your GD version is smaller than this, compress option to C<out()>
+will be silently ignored.
+
+=item [GD] setThickness
+
+setThickness implemented in GD v2.07. If your GD version is smaller
+than that and you set thickness option, your code will probably C<die>.
+
 =back
 
 =head1 BUGS
 
-=over 4
+=over 8
 
 =item Image::Magick bug
 
@@ -498,6 +602,10 @@ for GD::SecurityImage.
 
 =item GD bug
 
+=over 4
+
+=item path bug
+
 libgd and GD.pm does not like relative paths and paths that have spaces
 in it. If you pass a font path that is not an B<exact path> or a path that
 have a space in it, you may get an empty image. 
@@ -506,12 +614,14 @@ To check if the module failed to find the ttf font (when using C<GD>), a new
 method added: C<gdbox_empty()>. It must be called after C<create()>:
 
    $image->create;
-   die "Error loading ttf font for GD!" if $image->gdbox_empty;
+   die "Error loading ttf font for GD: $@" if $image->gdbox_empty;
 
 C<gdbox_empty()> always returns false, if you are using C<Image::Magick>.
 
 =back
 
+=back
+
 Contact the author if you find any bugs. You can also send requests.
 
 =head1 AUTHOR
