@@ -2,12 +2,23 @@ package GD::SecurityImage::Magick;
 # GD method emulation class for Image::Magick
 use strict;
 use vars qw[$VERSION];
-use constant IM_WIDTH    => 4;
-use constant IM_ASCENDER => 2;
+
+use constant X_PPEM       => 0; # character width 
+use constant Y_PPEM       => 1; # character height
+use constant ASCENDER     => 2; # ascender
+use constant DESCENDER    => 3; # descender
+use constant WIDTH        => 4; # text width
+use constant HEIGHT       => 5; # text height
+use constant MAX_ADVANCE  => 6; # maximum horizontal advance
+
+use constant ANGLE        => -2;
+use constant CHAR         => -1;
+
+use constant MAX_COMPRESS => 100;
 
 use Image::Magick;
 
-$VERSION = "1.12";
+$VERSION = "1.32";
 
 sub gdbox_empty {0} # fake method for GD compatibility.
 
@@ -27,6 +38,7 @@ sub init {
       $self->{image}->Read('null:' . $bg);
       $self->{image}->Set(background => $bg);
       $self->{MAGICK} = {strokewidth => 0.6};
+      $self->setThickness($self->{thickness}) if $self->{thickness};
 }
 
 sub out {
@@ -37,7 +49,18 @@ sub out {
       my %g = map {$_, 1} $self->{image}->QueryFormat;
       $type = $opt{force} if exists $g{$opt{force}};
    }
+   # compression JPEG  LosslessJPEG  Zip
    $self->{image}->Set(magick => $type);
+   if ($opt{'compress'} and $type =~ m[^(png|jpeg)$]) {
+      #if ($type eq 'gif') {
+      #   $self->{image}->Set(compression => 'LZW');
+      #}
+      if($type eq 'png') {
+         $opt{'compress'} = MAX_COMPRESS;
+         $self->{image}->Set(compression => 'Zip');
+      }
+      $self->{image}->Set(quality => $opt{'compress'});
+   }
    return $self->{image}->ImageToBlob, $type, $self->{_RANDOM_NUMBER_};
 }
 
@@ -46,22 +69,38 @@ sub insert_text {
    my $self   = shift;
    my $method = shift; # not needed with Image::Magick (always use ttf)
    my $key    = $self->{_RANDOM_NUMBER_}; # random string
-   my @metric = $self->{image}
-                     ->QueryFontMetrics(font      => $self->{font},
-                                        text      => $key,
-                                        pointsize => $self->{ptsize});
-   # Text is not in the middle. There is a minimal error placing the text.
-   # Some manual altering is required unless there is a better way...
-   my $x = ($self->{width}  - $metric[IM_WIDTH]   ) / 2;
-   my $y = ($self->{height} + $metric[IM_ASCENDER]) / 2;
-
-   $self->{image}->Annotate(font      => $self->{font},
-                            encoding  => 'UTF-8',
-                            text      => $key,
-                            pointsize => $self->{ptsize},
-                            fill      => $self->rgbx($self->{_COLOR_}{text}),
-                            x         => $x,
-                            y         => $y );
+   my $info   = sub {$self->{image}->QueryFontMetrics(font => $self->{font}, text => shift, pointsize => $self->{ptsize})};
+   my %same   = (font      => $self->{font},
+                 encoding  => 'UTF-8',
+                 pointsize => $self->{ptsize},
+                 fill      => $self->rgbx($self->{_COLOR_}{text}),
+   );
+   if ($self->{scramble}) {
+      my $space = [$info->(' '), 0, ' ']; # get " " parameters
+      my @randomy;
+      my $sy = $space->[ASCENDER] || 1;
+      push(@randomy,  $_, - $_) foreach $sy/2, $sy/4, $sy/8;
+      my @char;
+      foreach (split //, $key) {
+         push @char, [$info->($_), $self->random_angle, $_], $space, $space, $space;
+      }
+      my $total = 0;
+         $total += $_->[WIDTH] foreach @char;
+      foreach my $magick (@char) {
+         $total -= $magick->[WIDTH] * 2;
+         $self->{image}->Annotate(text   => $magick->[CHAR],
+                                  x      =>  ($self->{width}  - $total - $magick->[WIDTH]   ) / 2,
+                                  y      => (($self->{height}          + $magick->[ASCENDER]) / 2) + $randomy[int rand @randomy],
+                                  rotate => $magick->[ANGLE],
+                                  %same);
+      }
+      # stretch=>{Normal, UltraCondensed, ExtraCondensed, Condensed, SemiCondensed, SemiExpanded, Expanded, ExtraExpanded, UltraExpanded}
+   } else {
+      my @metric = $info->($key);
+      my $x = ($self->{width}  - $metric[WIDTH]   ) / 2;
+      my $y = ($self->{height} + $metric[ASCENDER]) / 2;
+      $self->{image}->Annotate(text => $key, x => $x, y => $y, %same);
+   }
 }
 
 sub setPixel {
@@ -126,6 +165,12 @@ sub arc {
       );
 }
 
+sub setThickness {
+   my $self = shift;
+   my $thickness = shift;
+   $self->{MAGICK}{strokewidth} = $thickness * $self->{MAGICK}{strokewidth} if $thickness;
+}
+
 1;
 
 __END__
